<?php
/**************************************************************************\
* eGroupWare                                                               *
* http://www.egroupware.org                                                *
* --------------------------------------------                             *
*  This program is free software; you can redistribute it and/or modify it *
*  under the terms of the GNU General Public License as published by the   *
*  Free Software Foundation; either version 2 of the License, or (at your  *
*  option) any later version.                                              *
\**************************************************************************/

/**
 * Implementa o design pattern factory para o mdulo Workflow
 * @author Carlos Eduardo Nogueira Gonalves
 * @author Sidnei Augusto Drovetto Jr. - drovetto@gmail.com
 * @version 1.1
 * @package Workflow
 * @license http://www.gnu.org/copyleft/gpl.html GPL
 */
class Factory
{
	/**
	 * @var array $_cache Cache dos objetos
	 * @access private
	 */
	private $_cache;

	/**
	 * @var array $classPath Caminho de busca das classes
	 * @access protected
	 */
	protected $classPath;

	/**
	 * @var array $preffix Prefixo para o nome das classes
	 * @access protected
	 */
	protected $preffix;

	/**
	 * @var array $suffix Sufixo para o nome das classes
	 * @access protected
	 */
	protected $suffix;

	/**
	 * Construtor da classe
	 * @return object
	 * @access public
	 */
	function Factory()
	{
		$this->_cache = array();
		$this->classPath = dirname(__FILE__);
		$this->preffix = 'class.';
		$this->suffix = '.inc.php';
	}

	/**
	 * Importa os arquivos da classe
	 * @param string $input Nome da classe
	 * @access private
	 * @return void
	 */
	private function _import($input)
	{
		$file = $this->classPath . '/' . $this->preffix . $input . $this->suffix;
		if (@file_exists($file))
			require_once $file;
		else
			trigger_error("O arquivo " . $file . " no existe.", E_USER_ERROR);
	}

	/**
	 * Armazena referncias aos objetos recm criados
	 * @param string $key O nome da classe
	 * @param object &$item A instncia da classe
	 * @return void
	 * @access private
	 */
	private function _setEntry($key, &$item)
	{
		$this->_cache[$key] = &$item;
	}

	/**
	 * Retorna a referncia ao objeto armazenado
	 * @param string $key Nome da classe
	 * @return object
	 * @access private
	 */
	private function &_getEntry($key)
	{
		return $this->_cache[$key];
	}

	/**
	 * Verifica se uma determinada classe possui objeto armazenado no cache da Factory
	 * @param string $key Nome da classe
	 * @return boolean
	 * @access public
	 */
	function isEntry($key)
	{
		return ($this->_getEntry($key) !== null);
	}

	/**
	 * Retorna objetos armazenados ou cria novos objetos utilizando o pattern singleton
	 * @param string $className Nome da classe
	 * @return object
	 * @access public
	 */
	function &getInstance()
	{
		if (func_num_args() > 0)
		{
			$args = func_get_args();
			if ($this->isEntry($args[0]))
			{
				return($this->_getEntry($args[0]));
			}
			else
			{
				$instance = call_user_func_array(array(&$this, 'newInstance'), $args);
				$this->_setEntry($args[0], &$instance);
				return $instance;
			}
		}
	}

	/**
	 * Cria novos objetos
	 * @param string $className Nome da classe
	 * @return object
	 * @access public
	 */
	function &newInstance()
	{
		if (func_num_args() > 0)
		{
			$args = func_get_args();
			$className = $args[0];
			array_splice($args, 0, 1);
			$this->_import($className);

			$reflectionObj = new ReflectionClass($className);
			return $reflectionObj->newInstanceArgs($args);
		}
	}

	/**
	 * Retorna objetos armazenados ou cria novos objetos utilizando o pattern singleton (somente para mdulos externos)
	 * @param string $moduleName O nome do mdulo
	 * @param string $className Nome da classe
	 * @return object
	 * @access public
	 */
	function &getForeignInstance()
	{
		if (func_num_args() > 1)
		{
			$args = func_get_args();

			$oldClassPath = $this->classPath;
			$this->classPath .= '/../../' . $args[0] . '/inc';

			array_splice($args, 0, 1);
			$instance = call_user_func_array(array(&$this, 'getInstance'), $args);
			$this->classPath = $oldClassPath;
			return $instance;
		}
	}

	/**
	 * Cria novos objetos (somente para mdulos externos)
	 * @param string $moduleName O nome do mdulo
	 * @param string $className Nome da classe
	 * @return object
	 * @access public
	 */
	function &newForeignInstance()
	{
		if (func_num_args() > 1)
		{
			$args = func_get_args();

			$oldClassPath = $this->classPath;
			$this->classPath .= '/../../' . $args[0] . '/inc';

			array_splice($args, 0, 1);
			$instance = call_user_func_array(array(&$this, 'newInstance'), $args);
			$this->classPath = $oldClassPath;
			return $instance;
		}
	}
}
?>